/* Font size css tags to be used across project */

.txt-10 {
    font-size: 10px;
}

.txt-12 {
    font-size: 12px;
}

.txt-14 {
    font-size: 14px;
}

.txt-16 {
    font-size: 16px;
}

.txt-20 {
    font-size: 20px;
}
