.suggestion-box-container {
    background-color: white;

    display: flex;
    flex-direction: column;
    
    width: 300px;
}

.suggestion-box-selected-function {
    background-color: #F1F1F1;

    padding-top: 10px;
    padding-right: 5px;
    padding-left: 15px;
}

.suggestion-box-selected-description {
    background-color: #F1F1F1;
    font-size: 10px;
    color: grey;

    padding-top: 3px;
    padding-right: 5px;
    padding-left: 15px;
    padding-bottom: 10px;
}

.suggestion-box-function {
    padding-top: 10px;
    padding-right: 5px;
    padding-left: 15px;
    padding-bottom: 5px;
}