
/* For labeling selection boxes, etc */
.modal-element-label {
    text-align: left;
    margin-bottom: 2px;
}

/* For any error in the modal (e.g. the user input an invalid value) */
.modal-error {
    text-align: left;
    color: red
}

/* When adding a value to a list, make the add text this */
.modal-add {
    text-align: left;
    color: #0081DE;
}

/* Style any input box in a modal with this */
.modal-input {
    width: 100%;
    height: 30px;
    background-color: #ebebeb;
    font-size: 12px;
    color: #343434;

    border-color: #343434;
    border-style: solid;
    border-width: 1px;
    border-radius: 2px;

    box-sizing: border-box;
    padding-left: 5px;
}