
.save-analysis-label {
    margin-bottom: 2px;
    text-align: left;
}

.save-analysis-analysis-selection-container {
    margin-top: 5px;
    margin-bottom: 15px;
}

.save-analysis-analysis-selection {
    border: 1px solid #DDDDDD;
    border-radius: 5px;
    background-color: #F8F8F8;
    height: 60px;
    overflow: auto;
    text-align: left;
    width:  100%;
}

.save-analysis-analysis-name {
    box-sizing: border-box;
    padding-left: 5px;
}

.save-analysis-name-error {
    color: #ED4747;
    text-align: left;
    margin-top: 5px;
}