/* height css tags to be used across project
   in the format: .h-{number} where number can be any multiple of 5 */

.h-5 {
    height: 5vw;
}

.h-8 {
    height: 8vw;
}

.h-10 {
    height: 10vw;
}

.h-15 {
    height: 15vw;
}

.h-20 {
    height: 20vw;
}

.h-25 {
    height: 25vw;
}

.h-30 {
    height: 30vw;
}

.h-35 {
    height: 35vw;
}

.h-40 {
    height: 40vw;
}

.h-45 {
    height: 45vw;
}

.h-50 {
    height: 50vw;
}

.h-55 {
    height: 55vw;
}

.h-60 {
    height: 60vw;
}

.h-65 {
    height: 65vw;
}

.h-70 {
    height: 70vw;
}

.h-75 {
    height: 75vw;
}

.h-80 {
    height: 80vw;
}

.h-85 {
    height: 85vw;
}

.h-90 {
    height: 90vw;
}

.h-95 {
    height: 95vw;
}

.h-100 {
    height: 100vw;
}