/* Margin css tags to be used across project */

.mt-0 {
    margin-top: 0vh;
}

.mt-p5 {
    margin-top: .5vh;
}

.mt-1 {
    margin-top: 1vh;
}

.mt-2 {
    margin-top: 2vh;
}

.mt-4 {
    margin-top: 4vh;
}

.mt-6 {
    margin-top: 6vh;
}

.mt-10 {
    margin-top: 10vh;
}

.mb-0 {
    margin-bottom: 0vh;
}

.mb-p5 {
    margin-bottom: .5vh;
}

.mb-1 {
    margin-bottom: 1vh;
}

.mb-2 {
    margin-bottom: 2vh;
}

.mb-4 {
    margin-bottom: 4vh;
}

.mb-6 {
    margin-bottom: 6vh;
}

.mb-10 {
    margin-bottom: 10vh;
}

.ml-0 {
    margin-left: 0vw;
}

.ml-p5 {
    margin-left: .5vw;
}

.ml-1 {
    margin-left: 1vw;
}

.ml-2 {
    margin-left: 2vw;
}

.ml-4 {
    margin-left: 4vw;
}

.ml-6 {
    margin-left: 6vw;
}

.ml-10 {
    margin-left: 10vw;
}

.mr-0 {
    margin-right: 0vw;
}

.mr-p5 {
    margin-right: .5vw;
}

.mr-1 {
    margin-right: 1vw;
}

.mr-2 {
    margin-right: 2vw;
}

.mr-3 {
    margin-right: 3vw;
}

.mr-4 {
    margin-right: 4vw;
}

.mr-6 {
    margin-right: 6vw;
}

.mr-10 {
    margin-right: 10vw;
}

.ma-0 {
    margin-left: 0vw;
    margin-right: 0vw;
    margin-top: 0vw;
    margin-bottom: 0vw;
}

.ma-p5 {
    margin-left: .5vw;
    margin-right: .5vw;
    margin-top: .5vw;
    margin-bottom: .5vw;
}

.ma-1 {
    margin-left: 1vw;
    margin-right: 1vw;
    margin-top: 1vw;
    margin-bottom: 1vw;
}

.ma-2 {
    margin-left: 2vw;
    margin-right: 2vw;
    margin-top: 2vw;
    margin-bottom: 2vw;
}

.ma-4 {
    margin-left: 4vw;
    margin-right: 4vw;
    margin-top: 4vw;
    margin-bottom: 4vw;
}

.ma-6 {
    margin-left: 6vw;
    margin-right: 6vw;
    margin-top: 6vw;
    margin-bottom: 6vw;
}

.ma-10 {
    margin-left: 10vw;
    margin-right: 10vw;
    margin-top: 10vw;
    margin-bottom: 10vw;
}




