.group-modal-sheet-selection {
    display: flex;
    flex-flow: column;
    justify-content: space-between;
    text-align: left;

    margin-bottom: 2px;
    color: #929292;
}


.group-modal-label {
    margin-bottom: 2px;
    text-align: left;
}


.group-modal-column-selection-container {
    margin-top: 10px;
}

.group-modal-column-selection {
    border: 1px solid #DDDDDD;
    border-radius: 5px;
    background-color: #F8F8F8;
    height: 60px;
    overflow: auto;
    text-align: left;
    width:  100%;
}

.group-modal-dropdown {
    border: 1px solid #DDDDDD;
    border-radius: 5px;

    height: 25px;

    color: #343434;
    background-color: #F8F8F8;
    font-size: 14px;

    width: 100%;
    padding-left: 5px;
}

.group-modal-value-mapping-container {
    height: 100px;
    overflow: auto;
}

.group-modal-value-mapping {
    display: flex;
    justify-content: start;
}

.group-modal-value-mapping-key {
    width: 50%;
}

.group-modal-value-mapping-value {
    margin-left: 1vh;
    width: 40%;
}

.group-modal-add-value {
    display: flex;
    color: #0081DE;
}


.group-modal-key-error {
    color: #ED4747;
    text-align: left;
    margin-bottom: 5px;
}