.mito-container {
    display: flex;
    flex-direction: column;
}

/* shade toolbar grey and set to the length of the mito sheet */
.toolbar {
    background-color: #C2C2C2;
    width: 100%;
}

/* Note this only works with one div, p, etc. at a time. */
.vertical-align-content {
    display: flex;
    flex-direction: column;
    justify-content: center;
}    

.vertical-line {
    border-left: 2px solid #999696;
    height: 25px;
    margin-left: 10px;
    margin-right: 10px;
}

.sidebar{
    /* Render above other divs */
    z-index: 2;
    position: absolute;

    /* Make it a sidebar on the right side */
    width: 300px;
    align-self: end;
    background-color: white;
    border-radius: 2px;
    border: 1px solid black;

    /* make the sidebar scroll hidden so that we can preserve the header at the top*/
    overflow: hidden;

    /* Add a drop shadow */
    box-shadow: 0 4px 8px 0 rgba(0, 0, 0, 0.2), 0 6px 20px 0 rgba(0, 0, 0, 0.19);

    margin-top: 50px;
    height: 350px;
}

.sheet-tab-bar {
    display: flex;
}