.mito-sheet {
  width: 100%;
  height: 250px; /* modify this to increase the size of the sheet. update in utils/gridstyling also */
  padding: 0px 2px;

  /* Bound the size, and scroll */
  overflow-y: auto;

  background-color: rgb(253, 253, 253);

  /* border-box to set the border inside the div, so it doesn't get chopped */
  box-sizing: border-box; 
  -moz-box-sizing: border-box; /* Firefox */
  -webkit-box-sizing: border-box; /* Older Webkit browsers */

  /* Put a price on emotion... */
  border: 1px solid;
  border-radius: 2px 2px 0px 0px;
}

.mito-codeblock {
  width: 100%;
  height: 50px;
  padding: 0px 2px;

  background-color: rgb(194, 194, 194);

  /* border-box to set the border inside the div, so it doesn't get chopped */
  box-sizing: border-box; 
  -moz-box-sizing: border-box; /* Firefox */
  -webkit-box-sizing: border-box; /* Older Webkit browsers */

  /* We'll be a fine line (and we don't double up on the top) ... */
  border-right: 1px solid;
  border-bottom: 1px solid;
  border-left: 1px solid;
  border-radius: 0px 0px 2px 2px;
}