
.function-documentation-function-name {
    margin-top: 1px;
    margin-bottom: 1px;
}

.function-documentation-example-list {
    margin-top: 0;
    list-style: none;
    padding-left: 0;
}

.function-documentation-example {
    margin-top: 2px;
    margin-bottom: 2px;
}

.blue-text {
    color: #0081DE;
}