.formula-bar-container {
    height: 30px;
    width: 100%;
    background-color: #ebebeb;

    font-family: Georgia;
}

.formula-bar {
    display: flex;
    flex-direction: row;
    justify-content: left;
    text-align: center;
    align-content: center;
}

.fx-text {
    font-size: 16px;
    margin-left: 10px;
}

.formula-bar-input {
    font-size: 16px;
    text-align: left;

    /* make input blend in with formula bar */
    background-color: #ebebeb;
    border-radius: 0;
    border-style: none;
    outline: none;
}
