
.merge-modal-label {
    margin-bottom: 2px;
    text-align: left;
}

.merge-modal-sheet-dropdown {
    border: 1px solid #DDDDDD;
    border-radius: 5px;

    height: 25px;

    color: #343434;
    background-color: #F8F8F8;
    font-size: 14px;

    width: 100%;
    padding-left: 5px;
}

.merge-modal-key-dropdown {
    height: 25px;

    border: 1px solid #DDDDDD;
    border-radius: 5px;
        
    color: #343434;
    background-color: #F8F8F8;
    font-size: 14px;

    width: 100%;
    padding-left: 5px;
}

.merge-modal-column-selection-container {
    margin-top: 5px;
    margin-bottom: 15px;
}

@media only screen and (max-width: 1250px) {
    /* 
        This handles the small merge modal.
    */

    .merge-modal-sheet-and-key {
        display: flex;
        flex-flow: column;
        text-align: left;
    
        margin-bottom: 2px;
        color: #929292;
    }

    .merge-modal-sheet {
        width: 100%;
    }

    .merge-modal-key {
        width: 100%;
    }

    .merge-modal-column-selection {
        border: 1px solid #DDDDDD;
        border-radius: 5px;
        background-color: #F8F8F8;
        height: 20px;
        overflow: auto;
        text-align: left;
        width:  100%;
    }
}

@media only screen and (min-width: 1250px) {

    /* This handles the large merge modal */

    .merge-modal-sheet-and-key {
        display: flex;
        flex-flow: row;
        justify-content: space-between;
        text-align: left;
    
        margin-bottom: 2px;
        color: #929292;
    }

    .merge-modal-sheet {
        width: 60%;
    }

    .merge-modal-key {
        width: 38%;
        margin-left: 2%;
    }
    
    .merge-modal-column-selection {
        border: 1px solid #DDDDDD;
        border-radius: 5px;
        background-color: #F8F8F8;
        height: 60px;
        overflow: auto;
        text-align: left;
        width:  100%;
    }
}