/* Padding css tags to be used across project */


.pt-0 {
    padding-top: 0vh;
}

.pt-p5 {
    padding-top: .5vh;
}

.pt-1 {
    padding-top: 1vh;
}

.pt-2 {
    padding-top: 2vh;
}

.pt-4 {
    padding-top: 4vh;
}

.pt-6 {
    padding-top: 6vh;
}

.pb-0 {
    padding-bottom: 0vh;
}

.pb-p5 {
    padding-bottom: .5vh;
}

.pb-1 {
    padding-bottom: 1vh;
}

.pb-2 {
    padding-bottom: 2vh;
}

.pb-4 {
    padding-bottom: 4vh;
}

.pb-6 {
    padding-bottom: 6vh;
}

.pl-0 {
    padding-left: 0vw;
}

.pl-p5 {
    padding-left: .5vw;
}

.pl-1 {
    padding-left: 1vw;
}

.pl-2 {
    padding-left: 2vw;
}

.pl-4 {
    padding-left: 4vw;
}

.pl-6 {
    padding-left: 6vw;
}

.pr-0 {
    padding-right: 0vw;
}

.pr-p5 {
    padding-right: .5vw;
}

.pr-1 {
    padding-right: 1vw;
}

.pr-2 {
    padding-right: 2vw;
}

.pr-3 {
    padding-right: 3vw;
}

.pr-4 {
    padding-right: 4vw;
}

.pr-6 {
    padding-right: 6vw;
}

.pa-0 {
    padding-left: 0vw;
    padding-right: 0vw;
    padding-top: 0vw;
    padding-bottom: 0vw;
}

.pa-p5 {
    padding-left: .5vw;
    padding-right: .5vw;
    padding-top: .5vw;
    padding-bottom: .5vw;
}

.pa-1 {
    padding-left: 1vw;
    padding-right: 1vw;
    padding-top: 1vw;
    padding-bottom: 1vw;
}

.pa-2 {
    padding-left: 2vw;
    padding-right: 2vw;
    padding-top: 2vw;
    padding-bottom: 2vw;
}

.pa-4 {
    padding-left: 4vw;
    padding-right: 4vw;
    padding-top: 4vw;
    padding-bottom: 4vw;
}

.pa-6 {
    padding-left: 6vw;
    padding-right: 6vw;
    padding-top: 6vw;
    padding-bottom: 6vw;
}