
/* 
    we add the overflow: hidden property to the AgGrid header cell to ensure that
    the column header does not overflow into the next cell. We have to add it here
    because it was not working properly when just adding it to the .column-header-cell
*/
.ag-header-cell {
    overflow: hidden;
}

.column-header-cell {
    display: flex;
    flex-flow: row; /* make space for the header and the filter icon */
    justify-content: space-between;
    align-items: center; /* make the header text and filter be aligned vertically */ 
}

.column-header-header-container {
    white-space: normal; /* suggest to use by ag-grid if we are wrapping text */ 
    word-break: break-all;
    margin-top: 5px;
    margin-bottom: 5px;
}

.column-header-column-header {
    display: inline-flex;
    flex-flow: row wrap; /* wrap the column header to make the word spans wrap to a new line*/
}

.column-header-column-header:hover {
    text-decoration: underline;
}
 
