.tab {
    /* set background color and border of sheet tab*/
    background: #FDFDFD;
    border: 1px solid #C2C2C2;
    box-sizing: border-box;

    /* set width of sheet tab to just divide up the space all the tabs to fit in */
    flex-grow: 1;
    height: 35px;

    /* ensure the text is relatively centered*/
    text-align: center;
    padding-top: 3px;
}

.tab-text {
    display: flex;
    justify-content: center;
}

.tab-sheet-name {
    margin-right: 5px;
}

.tab-sheet-shape {
    color: grey;
}

/* add a mito blue bar to the bottom of the selected sheet tab */
.selected-tab-indicator {
    border: 2px solid #0081DE;
    background-color: #0081DE;
}