

.documentation-basic-example-title {
    /* The title at the top of the page */
    font-size: 20px;
    text-decoration: underline;
}

.documentation-basic-example-section-title {
    /* A title that defines a new section, and so wants some space! */
    font-size: 20px;
    margin-top: 10px;
    text-decoration: underline;
}

.documentation-basic-example-python-code {
    /* This makes a box for any Python code to include in the documentation */

    border: 1.5px solid #0081DE;
    border-radius: 5px;

    margin-top: 5px;
    padding: 5px;
}

.documentation-basic-example-python .tooltip {
    /* the tooltip should default to hidden*/
    visibility: hidden;
    background-color: #343434;;
    color: #fff;
    text-align: center;
    border-radius: 6px;
    padding: 5px 0;
  
    font-size: 12px;
  
    width: 120px;
    top: 10%;
    margin-left: 75px; /* Center the item in the sidebar */
  
    /* Position the tooltip */
    z-index: 1;
  
    transition: 0s visibility;
}
  
.documentation-basic-example-python:hover .tooltip {
    /* when the toolbar item is hovered over, display the tooltip*/
    visibility: visible;
    transition-delay:.3s;
}

.documentation-basic-example-instructions-list {
    margin-top: 0;
    padding-left: 0px;
}

.documentation-basic-example-instructions-list-element {
    /* A list of instructions */
    padding-top: 5px;
    padding-bottom: 10px;
    display: flex;
    justify-content: space-between;
}

.documentation-basic-example-instructions-list-element-content {
    /* Content in a list of instructions in documentation; usually a numbered step, 
    and some content */
    padding-left: 5px;
    display: flex;
    flex-direction: column;
}

.documentation-basic-example-instructions-list-element-content-centered {
    /* 
        In a list of instructions in documentation, this centers content
        in the center of the documentation sidebar. 

        This maybe is too long for a name, lol
     */
    align-self: center;
}