/* Width css tags to be used across project 
   in the format: .w-{number} where number can be any multiple of 5 */


.w-5 {
    width: 5vw;
}

.w-8 {
    width: 8vw;
}

.w-10 {
    width: 10vw;
}

.w-15 {
    width: 15vw;
}

.w-20 {
    width: 20vw;
}

.w-25 {
    width: 25vw;
}

.w-30 {
    width: 30vw;
}

.w-35 {
    width: 35vw;
}

.w-40 {
    width: 40vw;
}

.w-45 {
    width: 45vw;
}

.w-50 {
    width: 50vw;
}

.w-55 {
    width: 55vw;
}

.w-60 {
    width: 60vw;
}

.w-65 {
    width: 65vw;
}

.w-70 {
    width: 70vw;
}

.w-75 {
    width: 75vw;
}

.w-80 {
    width: 80vw;
}

.w-85 {
    width: 85vw;
}

.w-90 {
    width: 90vw;
}

.w-95 {
    width: 95vw;
}

.w-100 {
    width: 100vw;
}

.w-90p {
    width: 90%;
}

.w-100p {
    width: 100%;
}