.filter-modal-centering-container {
    text-align: center;
    align-items: center;
}

.filter-modal-section-title {
    font-size: 14px;
    font-weight: bold;
    color: #343434;
    text-align: left;
    margin-bottom: 5px;
}

.filter-modal-center {
    text-align: center;
    align-items: center;
}

.filter-modal-condition-select {
    height: 15px;
    font-size: 12px;
    color: #343434;
    margin-top: .5px;
}

.input {
    border: 1px solid #DDDDDD;
    border-radius: 5px;

    height: 25px;

    color: #343434;
    background-color: #F8F8F8;
    font-size: 14px;

    width: 100%;
    padding-left: 5px;
}

.select {
    height: 25px;

    border: 1px solid #DDDDDD;
    border-radius: 5px;
        
    color: #343434;
    background-color: #F8F8F8;
    font-size: 14px;

    padding-left: 5px;
}

.filter-modal-value-input {
    margin-left: 10px;
    font-size: 12px;
    color: #343434;
    margin-top: .5px;
    width: 80%;
}

.filter-container {
    display: flex;
    margin-bottom: 5px;
}

.filter-condition-and-value-container {
    margin-left: 10px;
    display: flex;
    width: 70%;
    justify-content: space-between;
}

.filter-modal-add-value {
    display: flex;
    color: #0081DE;
}

.sort-buttons-div {
    display: flex;
    flex-direction: row wrap;
    justify-content: space-evenly;
}

.sort-button {
    padding-top: 1vh;
    padding-bottom: 1vh;
    padding-left: 2vw;
    padding-right: 2vw;

    font-size: 12px;
    background-color: #F8F8F8;
    border-radius: 3px;

    border-style: solid; /* get rid of shadow around button*/
}

.sort-button-selected{
    color: #0081DE;
    border-color: #0081DE;
}

.sort-button-unselected {
    color: #929292;
    border-color: #DDDDDD;
}

.filter-modal-section-spacer {
    margin-top: 15px;
}
