

.loading-indicator-container {
    /* Render above other divs */
    z-index: 2;
    position: absolute;

    /* Make it a sidebar on the right side */
    width: 100px;
    background-color: white;
    border-radius: 2px;
    border: 1px solid black;

    /* Add a drop shadow */
    box-shadow: 0 4px 8px 0 rgba(0, 0, 0, 0.2), 0 6px 20px 0 rgba(0, 0, 0, 0.19);

    /* Place it right inside the upper left corner of the sheet */
    margin-top: 85px;
    margin-left: 5px;
}

.loading-indicator-text {
    /* We have a margin, rather than a center, as the ...'s change the size of the text */
    margin-left: 20px;
}