.documentation-sidebar-container {
    overflow-y: hidden;

    /* give the documentation container a height for scrolling purposes */
    height: 350px;
}

.documentation-sidebar-header {
    display: flex;
    justify-content: space-between;
    border-bottom: 1px solid black;
    margin-bottom: 4px;
    background-color: #C4C4C4;

    /* give the documentation header a height for scrolling purposes */
    height: 35px;
}

.documentation-sidebar-header-title {
    font-size: 20px;
    margin-left: 10px;
    margin-top: 5px;
    margin-bottom: 5px;
}

.documentation-sidebar-header-back {
    display: flex;
    flex-direction: column;
    justify-content: space-around;
    margin-left: 10px;
}

.documentation-sidebar-header-back-and-title {
    /* This container holds the back and title buttons, when there is a back button! */
    display: flex;
}

.documentation-sidebar-header-close {
    font-size: 20px;
    margin-top: 5px;
    margin-right: 5px;
}

.documentation-sidebar-content {
    /* We make the content div scrollable! */
    overflow-y: scroll;

    /* give the documentation content a height for scrolling purposes */
    height: 315px;

    margin-left: 10px;
    margin-right: 10px;
}

.documentation-sidebar-content-function-list-title {
    font-size: 20px;
    text-decoration: underline;
}

.documentation-sidebar-content-function-list-section-title {
    font-size: 20px;
    margin-top: 10px;
    text-decoration: underline;
}


.documentation-sidebar-content-function-list {
    margin-top: 0;
    list-style: none;
    padding-left: 0px;
}

.documentation-sidebar-content-function-list-element {
    font-size: 14px;
    display: flex;
    margin-top: 10px;
}

.documentation-sidebar-content-function-list-icon {
    margin-right: 10px
}

/* So the user knows functions are  clickable! */
.documentation-sidebar-content-function-list-element:hover {
    text-decoration: underline;
}